from test.io.odysz.semantic.jprotocol import AnsonBody, AnsonMsg


class A:
    echo = "echo"
    inet = "inet"


class EchoReq(AnsonBody):

    def __init__(self, parent: AnsonMsg = None):
        super().__init__(parent)



import os
import unittest
from datetime import datetime

from src.anson.io.odysz.common import Utils


class UtilsTest(unittest.TestCase):
    def testFilePatterns(self):
        lines = [
            '@set jar_ver=0.7.1 # some comments',
            '@REM set version=0.7.0',
            '@set html_ver=0.1.5',
            'test_05_Utils.py',
            'UtilsTest().replaceFilePatterns()',
            str(datetime.now())]

        jar_ver = '0.7.2'
        html_ver= '0.1.6'
        version = '0.7.1'

        cwd = os.getcwd()
        version_file = os.path.join(cwd, 'test' if os.path.basename(cwd) != 'test' else '', 'res', '__version__.bat')
        Utils.writeline_nl(version_file, lines)

        Utils.update_patterns(version_file,
    {'@set jar_ver=[0-9\\.]+': f'@set jar_ver={jar_ver}',
             '@REM set version=[0-9\\.]+': f'@set version={version}',
             '@set html_ver=[0-9\\.]+': f'@set html_ver={html_ver}'})

        with open(version_file, 'r') as f:
            xlines = f.readlines()

            self.assertEqual(f'@set jar_ver={jar_ver} # some comments\n', xlines[0])
            self.assertEqual(f'@set version={version}\n', xlines[1])
            self.assertEqual(f'@set html_ver={html_ver}\n', xlines[2])

            for x in range(3, len(lines)):
                l, f = lines[x], xlines[x]
                self.assertEqual(l + '\n', f)


if __name__ == '__main__':
    unittest.main()
    t = UtilsTest()
    t.testFilePatterns()
    print('05: 0K')




import unittest

from src.anson.io.odysz.anson import class4Name, instanceof


class Instance4Test(unittest.TestCase):

    def testForwardRef(self):
        module_name = 'test.io.odysz.semantic.jprotocol'
        class_name = 'AnsonBody'
        cls = class4Name(module_name, class_name)
        self.assertEqual(f'{module_name}.{class_name}', cls.__type__)

        uri = 'test/instance-forward'
        obj = instanceof(cls, {"uri": uri})
        self.assertEqual(f'{module_name}.{class_name}', obj.__type__)
        self.assertEqual(uri, obj.uri)

        # Already warned about it:
        self.assertFalse(hasattr(obj, 'a'))
        # self.assertIsNone(obj.a)


if __name__ == '__main__':
    unittest.main()
    t = Instance4Test()
    t.testForwardRef()




from dataclasses import dataclass, field

from src.anson.io.odysz.anson import Anson


@dataclass
class ExtraData(Anson):
    s: str# = None
    i: int# = 0
    l: list# = field(default_factory=list)
    d: dict# = field(default_factory=dict)

    def __init__(self):
        super().__init__()
        self.s = None
        self.i = 0
        self.l = field(default_factory=list)
        self.d = field(default_factory=dict)




import sys
sys.stdout.reconfigure(encoding="utf-8")

import importlib.util
from dataclasses import dataclass, Field

import json
from enum import Enum
from importlib.util import find_spec
from numbers import Number
from typing import Union, Optional

from typing_extensions import get_args, get_origin

from .common import Utils

java_src_path: str = 'semanticshare'

def class4Name(m, clssn: str) -> type:
    """
    With returned class type:
        obj = cls()  # Adjust arguments as needed
        obj.__type__=f'{m}.{clssn}'
    :param m:
    :param clssn:
    :return class type:
    """
    
    def adjustMoudlename(m: str):
        def has_module(name: str):
            try:
                return find_spec(name) is not None
            except Exception as exp:
                print(exp)
            
        if len(m) == 0:
            return java_src_path if has_module(java_src_path) else ''
        
        return m if has_module(m) else f'{java_src_path}.{m}'
    
    module = __import__(adjustMoudlename(m), fromlist=[clssn])
    cls = getattr(module, clssn)
    cls.__type__=f'{m}.{clssn}'
    return cls

def getClass(_typ_: Optional[str]):
    if _typ_ is None:
        return None
    parts = _typ_.split('.')
    module = ".".join(parts[:-1])
    # m = __import__(module if module is not None else '__main__')
    # for comp in parts[1:]:
    #     m = getattr(m, comp)
    # return m
    return class4Name(module, parts[-1])


class AnsonField:
    def __init__(self, **kwargs):
        self.elemtype = kwargs.get('elemtype', None)
        antype = kwargs.get('type', None)
        self.antype = antype if not isinstance(antype, str) else getClass(antype)

    def isAnson(self):
        return self.antype is not None and type(self.antype) == type and issubclass(self.antype, Anson)


def _fields(anson, fromval):
    """
    Get all fields from an Anson type.
    :param anson: the object of Anson type, for finding its type.
    :param fromval:
    :return:
    """
    _FIELDS = '__dataclass_fields__'
    fds = getattr(type(anson), _FIELDS)
    return {it: AnsonField(type=f.type) for it, f in fds.items()}


class DataStruct:
    def __init__(self, **kwargs):
        self.isEnm = kwargs.get('enum', False)
        self.isLst = kwargs.get('lst', False)
        self.isObj = kwargs.get('obj', False)
        self.isStr = kwargs.get('str', False)
        self.isNum = kwargs.get('num', False)
        self.isBool= kwargs.get('bool', False)
        self.isAnson = kwargs.get('anson', False)
        self.ansoname = kwargs.get('ansontype', f'{__name__}.Anson')


def value_type(v):
    if v is None: return DataStruct()
    t = type(v)
    return DataStruct(anson = issubclass(t, Anson),
                      bool= t==bool,
                      lst = t==list,
                      obj = t==dict or t == object,
                      num = isinstance(v, Number),
                      str=t == str,
                      enum=isinstance(v, Enum),
                      ansontype=f'{t.__module__}.{t.__name__}')


def instanceof(clsname: Union[str, type], props: dict):
    cls = getClass(clsname) if type(clsname) == str else clsname
    try: obj = cls() 
    except Exception as e:
        print('Cannot create instance of', clsname, e)
        raise e
    fds = _fields(obj, None)
    missingAttrs = []

    for k, v in props.items():
        if k != 'type' and k not in fds:
            missingAttrs.append(k)
        setattr(obj, k, Anson.from_value(fds[k].antype if k in fds else None, v))

    if len(missingAttrs) > 0:
        Utils.warn(f'Missing attributes in {obj.__type__}: {missingAttrs}. Anson expect a __init__() initializing all the none default fields.')

    return obj


def parse_type_(obj) -> Union[str, None]:
    return obj['__type__'] if isinstance(obj, Anson) and hasattr(obj, '__type__') \
        else obj["type"] if isinstance(obj, dict) and 'type' in obj.keys() else None


def parse_forward(ref: Union[type, str]):
    return ref[0] if type(ref) == list else \
        get_args(ref)[0] if get_origin(ref) == list else \
        getClass(ref) if type(ref) == str else \
        ref


class JsonOpt:
    quotekey = True;

    def quoteK(self):
        return self.quotekey;


@dataclass
class Anson(dict):
    enclosinguardtypes = set()
    
    __type__: str
    '''ansons.antson.Anson'''

    def __init__(self):
        super().__init__()
        t = type(self)
        self.__type__ = f'{t.__module__}.{t.__name__}'

    def __setitem__(self, key, value):
        self.__dict__[key] = value

    def __getitem__(self, key):
        return self.__dict__[key]

    @staticmethod
    def toList_(lst: list, ind: int, beautify):
        return  '[' + ', '.join([e.toBlock_(ind + 0, beautify) if isinstance(e, Anson) \
               else f'"{e}"' if isinstance(e, str) and e is not None \
               else str(e) for e in lst]) + ']'

    @staticmethod
    def toValue_(v, ind: int, beautify: bool):
        vt = value_type(v)
        return 'null' if v is None \
            else f'"{v}"' if vt.isStr \
            else v.toBlock_(ind + 1, beautify, vt.ansoname) if vt.isAnson \
            else f'"{v.name}"' if vt.isEnm \
            else ('true' if v == True else 'false') if vt.isBool \
            else Anson.toList_(v, ind + 1, beautify) if vt.isLst \
            else Anson.toDict_(v, ind + 1, beautify) if vt.isObj \
            else str(v)

    @staticmethod
    def toDict_(dic: dict, ind: int, beautify):
        nl = '\n'
        return '{}' if len(dic) == 0 else \
            f'{"{" + nl if len(dic) > 1 else "{"}' + ',\n'.join(f'{" " * (ind * 2 + 2) if len(dic) > 1 else ""}' + f'"{k}": ' + Anson.toValue_(dic[k], ind, beautify) for k in dic) + '}' if beautify else \
            '{' + ','.join(f'"{k}": ' + Anson.toValue_(dic[k], ind + 1, beautify) for k in dic) + '}'

    def toBlock(self, beautify=True) -> str:
        '''
        Serialize.
        tip
        ===
        1. It is important to declare all the user classes the subclasses or Anson,
           and with a '@dataclass' annotation.
        2. If print() an instance of Anson, use anson.py3/utils.log() for safety,
           by which the fields declared but not created where not causing crashes.

        :param beautify: 
        :return: 
        '''
        return self.toBlock_(0, beautify)

    def toFile(self, path: str):
        with open(path, 'w+', encoding="utf-8") as jf:
            jf.write(self.toBlock(True))

    def toBlock_(self, ind: int, beautify, suggestype: type = None) -> str:
        myfds = _fields(self, None)
        s = '{\n' if beautify else '{'

        has_prvious = False

        for x, k in enumerate(self.__dict__):
            if '__type__' == k:
                tp = str(self['__type__']).removeprefix(java_src_path+'.')
                if has_prvious: s += ',\n' if beautify else ', '
                s += f'{" " * (ind * 2 + 2) if beautify else ""}"type": "{tp}"'
                has_prvious = True

            else:
                v = self[k]

                if k not in myfds:
                    if k != 'type':
                        Utils.warn("Field {0}.{1} is not defined in Anson or Semantics.py3, which is presenting in data object. Value ignored: {2}.",
                               str(self['__type__']), k, v)
                    continue

                if has_prvious: s += ',\n' if beautify else ','

                s += f'{" " * (ind * 2 + 2)}"{k}": ' if beautify else f'"{k}": '

                s += 'null' if v is None or isinstance(v, Field) \
                    else Anson.toValue_(v, ind, beautify)
                    # else f'"{v}"' if vt.isStr \
                    # else v.toBlock_(ind + 1, beautify, myfds[k].antype) if vt.isAnson \
                    # else f'"{v.name}"' if vt.isEnm \
                    # else Anson.toList_(v, ind + 1, beautify) if vt.isLst \
                    # else Anson.toDict_(v, ind + 1, beautify) if vt.isObj \
                    # else str(v)

                has_prvious = True
        return s + ('\n' if has_prvious and beautify else '') + (' ' * (ind * 2) + '}' if beautify else '}')

    @staticmethod
    def from_value(antype: Union[str, type, None], v):
        if v is None: return None
        vt = value_type(v)

        objtype = parse_type_(v)
        antype = objtype if objtype is not None else antype

        if type(antype) == str:
            try: antype = getClass(antype)
            except: pass

        # return Anson.from_obj(v, anf.antype) if vt.isAnson \
        # return instanceof(antype, v) if vt.isAnson or type(antype) == type and issubclass(antype, Anson) \
        #     else Anson.from_dict(v, antype) if vt.isObj \
        #     else Anson.from_list(v, parse_forward(antype)) if vt.isLst \
        #     else v
        antype = parse_forward(antype)
        return \
            Anson.from_list(v, antype) if vt.isLst else \
            instanceof(antype, v) if vt.isAnson or type(antype) == type and issubclass(antype, Anson) else \
            Anson.from_dict(v, antype) if vt.isObj else \
            v

    @staticmethod
    def from_dict(v: dict, eletype: Union[type, str, None]) -> dict:
        if eletype is None: return v

        d = {}
        # d, fds = {}, None
        # if eletype is str and issubclass(getClass(eletype), Anson):
        #     d = getClass(eletype)()
        #     fds = _fields(d, None)
        # elif eletype is type and issubclass(eletype, Anson):
        #     d = eletype()
        #     fds = _fields(d, None)

        for k in v:
            # d[k] = Anson.from_value(None if fds is None else fds[k].antype, v[k])
            d[k] = Anson.from_value(eletype, v[k])
        return d

    @staticmethod
    def from_list(v: list, eletype: str) -> list:
        # override field type with protocol package's info
        if not isinstance(v, list):
            Utils.warn(f'Expection a list instance, but got "{v}"')
            return [v]

        if len(v) > 0:
            _type_ = parse_type_(v[0])
            eletype = _type_ if _type_ is not None and len(_type_) > 0 else eletype
            # eletype = f'{java_src_path}.{eletype}'

        return 'null' if v is None else [Anson.from_value(eletype, x) for x in v]

    @staticmethod
    def from_obj(obj: dict, ansontype: Union[str, type]) -> Union['Anson', dict, None]:

        if obj is None: return None

        vt = value_type(obj)

        if vt.isAnson:
            raise TypeError('Not goes alone from_value()?')
            # antype = getClass(ansontype) if vt.ansoname is None else getClass(vt.ansoname)
            # return instanceof(antype, obj)
        elif vt.isObj:
            _t = parse_type_(obj)

            return instanceof(_t, obj) if _t is not None else instanceof(ansontype, obj) if ansontype is not None else obj

            # dicv = {} if _t is None and ansontype is None else getClass(_t)() if _t is not None else getClass(ansontype)()
            # fds = _fields(dicv, obj)
            # # if '__type__' not in fds and type(anson) != dict:
            # #     raise Exception(f'Class {type(anson)} has no field "__type__". Is it a subclass of Anson?')
            #
            # for jsonk in obj:
            #     k = '__type__' if jsonk == 'type' else jsonk
            #     if k != '__type__' and k not in fds:
            #         Utils.warn(f'Field ignored: {k}: {obj[k]}')  # TODO deserialize enum, e.g. MsgCode
            #         continue
            #
            #     dicv[k] = Anson.from_value(fds[k], obj[jsonk])
            # return dicv
        else:
            raise TypeError(f'Not here: {obj}')

    @staticmethod
    def from_json(jsonstr: str) -> 'Anson':
        obj = json.loads(jsonstr)
        v = Anson.from_envelope(obj)
        print(v, type(v))
        return v

    @staticmethod
    def from_file(fp: str) -> 'Anson':
        with open(fp, 'r', encoding='utf-8') as file:
            obj = json.load(file)
            return Anson.from_envelope(obj)

    @classmethod
    def java_src(cls, src_root: str = 'src'):
        """
        Example
        -------
        To deserialize type: io.oz.syn.AppSettings for Python class src.io.oz.syn.AppSetting
        from synode.py3, call::
        ```python
            Anson.java_src('src')

        :param src_root: e. g. 'src',
        :param requires
        """
        # if LangExt.len(requires) > 0:
        #     from importlib.metadata import distribution
        #     for req in requires:
        #         distribution(req)

        global java_src_path
        java_src_path = src_root

    @classmethod
    def from_envelope(cls, obj: dict):
        return Anson.from_obj(obj, obj['type'])
                # '.'.join([java_src_path, obj['type']]) if len(java_src_path) > 0 else obj['type'])


class AnsonException(Exception):
    type = "io.odysz.ansons.x.AnsonException"
    excode = 0
    err = ""

    def __init__(self, excode: int, template: str, *param: object):
        super().__init__()
        self.excode = excode
        self.err = template if param is None else template.format(param)



from dataclasses import dataclass
from enum import Enum

from src.anson.io.odysz.anson import Anson, JsonOpt

class MsgCode(Enum):
    """
    public enum MsgCode {ok, exSession, exSemantic, exIo, exTransct, exDA, exGeneral, ext };
    """
    ok = 'ok'
    exSession = 'exSession'
    exSemantics = 'exSemantics'
    exIo = 'exIo'
    exTransc = 'exTransac'
    exDA = 'exDA'
    exGeneral = 'exGeneral'
    ext = 'ext'


class Port(Enum):
    echo = 'echo.less'
    session = "login.serv"
    r = "r.serv"


@dataclass
class AnsonHeader(Anson):
    uid: str
    ssid: str
    iv64: str
    usrAct: list[str]
    ssToken: str

    def __init__(self, ssid = None, uid = None, token = None):
        super().__init__()
        self.ssid = ssid
        self.uid = uid
        self.ssToken = token


@dataclass
class AnsonMsg(Anson):
    body: list['AnsonBody']
    header: AnsonHeader
    port: Port
    code: MsgCode

    def __init__(self, p: Port = None):
        super().__init__()
        self.port = p
        self.body = []

    def Header(self, h: AnsonHeader) -> 'AnsonMsg':
        self.header = h
        return self

    # def Body(self, bodyItem: TAnsonBody) -> Self:
    def Body(self, bodyItem: 'AnsonBody') -> 'AnsonMsg':
        self.body.append(bodyItem)
        return self


@dataclass
class AnsonBody(Anson):
    uri: str
    a: str
    rs: dict
    m: str
    map: dict
    opts: JsonOpt
    addr: str
    version: str
    seq: int


    def __init__(self, parent: AnsonMsg = None):
        super().__init__()
        self.uri = None
        self.parent = parent
        Anson.enclosinguardtypes.add(AnsonMsg)

    def A(self, a: str) -> 'AnsonBody':
        self.a = a
        return self

@dataclass
class AnsonReq(AnsonBody):
    def __init__(self):
        super().__init__()
        self.a = None


@dataclass
class AnsonResp(AnsonBody):
    code: MsgCode
    parent: str

    def __init__(self):
        super().__init__()
        self.a = None
        self.code = MsgCode.ok




import fnmatch
import os
import re
import zipfile


def zip2(distzip, resources, exclude_patterns=[]):
    """
    example: zip2('registry-zsu.zip', {"zsu": "registry-deploy/*"}, ['*.zip'])
    :param distzip:
    :param resources:
    :param exclude_patterns:
    :return: None
    """
    def matches_patterns(filename, patterns):
        """
        Check if a filename matches any of the given patterns.

        Args:
            filename (str): The filename to check (e.g., "data.logs").
            patterns (list): List of patterns (e.g., ["*.logs", "*.dat"]).

        Returns:
            bool: True if the filename matches any pattern, False otherwise.
        """
        return any(fnmatch.fnmatch(os.path.basename(filename), pattern) for pattern in patterns)

    with zipfile.ZipFile(distzip, 'w', zipfile.ZIP_DEFLATED) as zipf:
        err = False
        # resources
        for rk, rv in resources.items():
            if "*" in rv:
                count = 0
                srcroot = re.sub('\\*$', '', rv.replace('\\', '/'))
                for pth, _dir, fs in os.walk(srcroot):
                    for file in fs:
                        if not matches_patterns(file, exclude_patterns):
                            file_path = os.path.join(pth, file)
                            relative_path = os.path.relpath(file_path, srcroot)
                            # print(file, file_path, pth, srcroot, relative_path)

                            visited = set()
                            while os.path.islink(file_path):
                                if file_path in visited:
                                    raise ValueError(f"Cycle detected in symbolic links at {relative_path}")
                                visited.add(file_path)
                                print(file_path, '->', os.path.realpath(file_path))
                                file_path = os.path.realpath(file_path)

                            relative_path = os.path.relpath(relative_path)
                            arcname = os.path.join(rk, relative_path)
                            zipf.write(file_path, arcname)
                            count += 1
                            print(f"Added to ZIP: {relative_path} as {arcname}")
                if count == 0:
                    err = True
                    raise FileNotFoundError(f'[ERROR] No files found in {rv}.')
            else:  # Handle single files (jserv.jar and exiftool.exe)
                file = rk if rv == '.' else rv
                if os.path.exists(file):
                    zipf.write(file, rk)
                    print(f"Added to ZIP: {file} as {rk}")
                else:
                    err = True
                    raise FileNotFoundError(f"[ERROR]: Resource '{rk}': '{file}' not found.")

    print(f'Created ZIP file successfully: {distzip}' if not err else 'Errors while making target (creaded zip file)')



'''
Created on 25 Oct 2019

@author: odys-z@github.com
'''
import os
import sys
from re import match
from typing import TextIO, Optional, TypeVar, Union


T = TypeVar('T')

passwd_allow_ext = ' @#!$%^&*()_+-='
'''
    allowed chars in addition to alpha numerics for password.
'''

class LangExt:
    '''
    Language helper
    '''

    def __init__(self, params):
        '''
        Constructor
        '''

    @staticmethod
    def isblank(s, regex=None):
        """
        ::
        
            self.assertTrue(LangExt.isblank(None))
            self.assertTrue(LangExt.isblank(''))
            self.assertTrue(LangExt.isblank(' '))
            self.assertTrue(LangExt.isblank('00', r'0+'))
            self.assertTrue(LangExt.isblank('00', r'0'))
            self.assertFalse(LangExt.isblank(' ', r'0'))
        :param s:
        :param regex:
        :return: is it taken as blank string
        """
        if (s == None):
            return True
        if isinstance(s, str):
            if regex == None:
                return len(s.strip()) == 0
            else:
                return match(regex, s) is not None
        return False

    @staticmethod
    def ifnull(a: T, b: T) -> T:
        return b if a is None else a

    @staticmethod
    def ifblank(a: str, b: str) -> str:
        return b if len(a) == 0 else a

    @classmethod
    def len(cls, obj):
        return 0 if obj is None else len(obj)

    @staticmethod
    def str(obj: Union[dict, list]):
        def quot(v) -> str:
            return f'"{v}"' if type(v) == str else f'"{v.toBlock()}"' if isinstance(v, Anson) else LangExt.str(v)
        from .anson import Anson
        if type(obj) == dict:
            s = '{'
            for k, v in obj.items():
                # s += f'{"" if len(s) == 1 else ",\n"}"{k}": "{LangExt.str(v)}"'
                SEP = ",\n"
                s += f'{"" if len(s) == 1 else SEP}"{k}": {quot(v)}'
            s += '}'
            return s
        elif type(obj) == list:
            s = '['
            # s += ", ".join(f'"{x}"' if type(x) == str else LangExt.str(x) for x in obj)
            s += ", ".join(quot(x) for x in obj)
            return s + ']'
        elif isinstance(obj, Anson):
            return obj.toBlock()
        else:
            return str(obj)

    @staticmethod
    def musteqs(a: str, b: str, msg = None):
        if a != b:
            from .anson import AnsonException
            raise AnsonException(0, f'{a} != {b}' if msg == None else msg)

    @staticmethod
    def only_wordextlen(likely: str, ext='', minlen = 0, maxlen = -1):
        if ext is None:
            ext = ''
        if maxlen >= 0 and len(likely) > maxlen:
            from .anson import AnsonException
            raise AnsonException(0, f'len {likely[0: 10]} > {maxlen}')

        if minlen > 0 and len(likely) < minlen:
            from .anson import AnsonException
            raise AnsonException(0, f'len {likely[0:10]} < {minlen}')

        for c in likely:
            if not c.isalnum() and c not in ext:
                from .anson import AnsonException
                raise AnsonException(0, f'Not allowed char: {c}')
        return True


    @staticmethod
    def only_wordtlen(likely: str, minlen=0, maxlen=-1):
        return LangExt.only_wordextlen(likely, minlen=minlen, maxlen=maxlen)

    @staticmethod
    def only_id_len(likely: str, ext='', minlen=0, maxlen=-1):
        '''
        Verify the *likely* string is only with chars of alphanumberic or anyof '`~!@#$%^&*_-+=:;,./'.
        :param likely: 
        :param ext: 
        :param minlen: 
        :param maxlen: 
        :return: verified
        '''
        return LangExt.only_wordextlen(likely,
                ext='`~!@#$%^&*_-+=:;,./' if ext is None else ext + '`~!@#$%^&*_-+=:;,./',
                minlen=minlen, maxlen=maxlen)

    @staticmethod
    def only_passwdlen(likely: str, minlen=0, maxlen=-1):
        '''
        String likely mus only an alphanumeric word and with length in between [minlen, maxlen].
        :param likely:
        :param minlen:
        :param maxlen:
        :return: likely
        '''
        return LangExt.only_wordextlen(likely, ext=passwd_allow_ext, minlen=minlen, maxlen=maxlen)


def log(out: Optional[TextIO], templt: str, *args):
    try:
        print(templt if LangExt.isblank(args) else templt.format(*args), file=out)
    except Exception as e:
        try: print(templt)
        except: pass
        try: print(args)
        except: pass
        try: print(e)
        except: pass
        print('If printing Anson subclasses, all their memebers must be initialized.', file=sys.stderr)


class Utils:
    def __init__(self, params):
        '''
        Constructor
        '''

    @staticmethod
    def logi(templt, *args):
        log(sys.stdout, templt, *args)

    @staticmethod
    def warn(templt, *args):
        log(sys.stderr, templt, *args)

    @staticmethod
    def get_os():
        """
        :return: Windows | Linux | macOS
        """
        if os.name == 'nt':
            return 'Windows'
        elif os.name == 'posix':
            if sys.platform.startswith('linux') or sys.platform.startswith('freebsd'):
              return 'Linux'
            elif sys.platform.startswith('darwin'):
                return 'macOS'
        return 'Unknown'

    @staticmethod
    def iswindows():
        return Utils.get_os() == 'Windows'

    @staticmethod
    def update_patterns(file, patterns: dict):
        """
        Update the version in a text file.

        Example
        -------
        ::

            Utils.update_patterns(version_file,
                {'@set jar_ver=[0-9\\.]+': f'@set jar_ver={jar_ver}',
                 '@REM set version=[0-9\\.]+': f'@set version={version}',
                 '@set html_ver=[0-9\\.]+': f'@set html_ver={html_ver}'})

        Args:
            file (str): Path to the JAR file.
            patterns (dict): Regular expression pattern, key, to replace with value.
        """
        import re
        print('Updating JAR version...', file)

        with open(file, 'r') as f:
            lines = f.readlines()

        cnt = 0
        # updated_content = re.sub(pattern, repl, content)
        for i, line in enumerate(lines):
            updated = set()
            for k, v in patterns.items():
                if re.search(k, line):
                    lines[i] = re.sub(k, v, line)
                    updated.add(k)
                    print('Updated line:', lines[i])
                    cnt += 1

                if len(updated) == len(patterns):
                    break

        with open(file, 'w') as f:
            f.writelines(lines)

        print(f'[{cnt / len(patterns)}] lines updated. Patterns updating finsiedh.', file)

        return None

    @classmethod
    def writeline_nl(cls, file: str, lines: list[str]):
        with open(file, 'w+') as f:
            for l in lines:
                f.write(l)
                f.write('\n')



from dataclasses import dataclass

from src.anson.io.odysz.anson import Anson


@dataclass
class SessionInf(Anson):
    uid: str
    pswd: str
    ssid: str
    ssToken: str

    def __init__(self, uid: str = None, pswd: str = None):
        super().__init__()
        self.uid = uid
        self.pswd = pswd
        self.ssid = None
        self.ssToken = None
